/*	$Id: mouse.c,v 1.8 1996/10/12 17:19:01 sandro Exp $	*/

/*
 * Copyright (c) 1996
 *	Sandro Sigala, Brescia, Italy.  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgement:
 *	This product includes software developed by Sandro Sigala.
 * 4. Neither the name of the author nor the names of any contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <time.h>

#include <vga.h>
#include <vgagl.h>
#include <vgamouse.h>

#include "mouse.h"
#include "mouse_arrow.c"

int wmouse_xmin = 0;
int wmouse_ymin = 0;
int wmouse_xmax = 0;
int wmouse_ymax = 0;
int wmouse_x = 0;
int wmouse_y = 0;
int wmouse_oldx = 0;
int wmouse_oldy = 0;
int wmouse_button = 0;
int wmouse_oldbutton = 0;
int wmouse_status = 0;
int wmouse_color_white = 15;
int wmouse_color_black = 0;
static int wmouse_saved = 0;
static int wmouse_save[PTR_WIDTH * PTR_HEIGHT];
static char *wmouse_pointer;

void wmouse_show(void);
void wmouse_hide(void);
void wmouse_update(void);
void wmouse_reset(void);

#define onscreen(x,y) \
	((x)>=wmouse_xmin && (x)<=wmouse_xmax && \
	 (y)>=wmouse_ymin && (y)<=wmouse_ymax)

void
wmouse_setpointer(char *pointer)
{
	if (wmouse_pointer == pointer)
		return;
	if (wmouse_status == 1)
		wmouse_hide();
	wmouse_pointer = pointer;
		wmouse_show();
}

static void
wmouse_hidep(int xpos, int ypos)
{
	int x, y;

	if (wmouse_status == 0)
		return;

	wmouse_status = 0;
	wmouse_saved = 1;

	xpos -= PTR_HOTSPOT_X(wmouse_pointer);
	ypos -= PTR_HOTSPOT_Y(wmouse_pointer);

	for (y = 0; y < PTR_HEIGHT; y++) {
		for (x = 0; x < PTR_WIDTH; x++) {
			if (onscreen(xpos + x, ypos + y)) {
				gl_setpixel(xpos + x, ypos + y,
					    wmouse_save[x + y * PTR_HEIGHT]);
			}
		}
	}
}

void
wmouse_hide(void)
{
	wmouse_hidep(wmouse_x, wmouse_y);
}

int
get_pointer_color(unsigned char *pointer, int x, int y)
{
	int c;
	if ((c = PTR_DATA(pointer, x, y)) == PTR_BLACK)
		return wmouse_color_black;
	else if (c == PTR_WHITE)
		return wmouse_color_white;
	else
		return -1;
}

void
wmouse_show(void)
{
	int x, y, xpos, ypos, c;

	if (wmouse_status == 1)
		return;

	wmouse_status = 1;

	if (wmouse_saved == 0)
		return;

	wmouse_saved = 0;

	xpos = wmouse_x - PTR_HOTSPOT_X(wmouse_pointer);
	ypos = wmouse_y - PTR_HOTSPOT_Y(wmouse_pointer);

	for (y = 0; y < PTR_HEIGHT; y++) {
		for (x = 0; x < PTR_WIDTH; x++) {
			if (onscreen(xpos + x, ypos + y)) {
				wmouse_save[x + y * PTR_HEIGHT] = gl_getpixel(xpos + x, ypos + y);
				if ((c = get_pointer_color(wmouse_pointer, x, y)) != -1)
					gl_setpixel(xpos + x, ypos + y, c);
			}
		}
	}
}

void
wmouse_update(void)
{
	mouse_update();

	wmouse_oldx = wmouse_x;
	wmouse_oldy = wmouse_y;
	wmouse_oldbutton = wmouse_button;

	wmouse_x = mouse_getx();
	wmouse_y = mouse_gety();
	wmouse_button = mouse_getbutton();

	if (wmouse_x < wmouse_xmin)
		wmouse_x = wmouse_xmin;
	if (wmouse_x > wmouse_xmax)
		wmouse_x = wmouse_xmax;
	if (wmouse_y < wmouse_ymin)
		wmouse_y = wmouse_ymin;
	if (wmouse_y > wmouse_ymax)
		wmouse_y = wmouse_ymax;

	if (wmouse_x != wmouse_oldx || wmouse_y != wmouse_oldy) {
		if (wmouse_status == 1)
			wmouse_hidep(wmouse_oldx, wmouse_oldy);
		wmouse_show();
	}
}

void
wmouse_update_window(int x0, int y0, int x1, int y1)
{
	mouse_update();

	wmouse_oldx = wmouse_x;
	wmouse_oldy = wmouse_y;
	wmouse_oldbutton = wmouse_button;

	wmouse_x = mouse_getx();
	wmouse_y = mouse_gety();
	wmouse_button = mouse_getbutton();

	if (wmouse_x < wmouse_xmin)
		wmouse_x = wmouse_xmin;
	if (wmouse_x > wmouse_xmax)
		wmouse_x = wmouse_xmax;
	if (wmouse_y < wmouse_ymin)
		wmouse_y = wmouse_ymin;
	if (wmouse_y > wmouse_ymax)
		wmouse_y = wmouse_ymax;
	if (wmouse_x < x0)
		wmouse_x = x0;
	if (wmouse_x > x1)
		wmouse_x = x1;
	if (wmouse_y < y0)
		wmouse_y = y0;
	if (wmouse_y > y1)
		wmouse_y = y1;

	if (wmouse_x != wmouse_oldx || wmouse_y != wmouse_oldy) {
		if (wmouse_status == 1)
			wmouse_hidep(wmouse_oldx, wmouse_oldy);
		wmouse_show();
	}
}

void
wmouse_reset(void)
{
	wmouse_xmin = 0;
	wmouse_ymin = 0;
	wmouse_xmax = WIDTH - 1;
	wmouse_ymax = HEIGHT - 1;
	wmouse_x = WIDTH / 2;
	wmouse_y = HEIGHT / 2;
	wmouse_oldx = wmouse_x;
	wmouse_oldy = wmouse_y;
	mouse_setposition(wmouse_x, wmouse_y);
	wmouse_pointer = &arrow_ptr[0];
}
